<?php

namespace Drupal\monarch_symlink\Controller;

use Drupal\Core\Url;
use Drupal\system\MenuInterface;
use Drupal\Core\Entity\EntityInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\HttpKernel\Event\RequestEvent;
use Symfony\Component\HttpKernel\HttpKernelInterface;
use Symfony\Component\HttpKernel\Event\ControllerEvent;
use Symfony\Component\HttpKernel\Event\ControllerArgumentsEvent;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Defines a controller to render a symlink.
 */
class SymlinkController extends SymlinkControllerBase {

  /**
   * Provides the symlink add form for a menu.
   *
   * @param \Drupal\system\MenuInterface $menu
   *   An entity representing a custom menu.
   *
   * @return array
   *   Returns the symlink creation form.
   */
  public function addSymlink(MenuInterface $menu) {
    $symlink = $this->entityTypeManager()
      ->getStorage('symlink')
      ->create([
        'menu_name' => $menu->id(),
      ]);
    return $this->entityFormBuilder()->getForm($symlink, 'add');
  }

  /**
   * The _title_callback for the page that renders a symlink.
   *
   * @param \Drupal\Core\Entity\EntityInterface $symlink
   *   The current symlink.
   *
   * @return string
   *   The page title.
   */
  public function title(EntityInterface $symlink) {
    return $symlink->getTitle();
  }

  /**
   * {@inheritdoc}
   */
  public function view(EntityInterface $symlink, $view_mode = 'full', $langcode = NULL) {
    $uri = $symlink->link->uri;

    try {
      $url = Url::fromUri($uri);
    }
    catch (\Exception $e) {
      // @TODO Log a message if target url is invalid.
      throw new NotFoundHttpException();
    }

    return $this->getUrlResponse($url->toString());
  }

  /**
   * Get the response object for a page based on a URL.
   *
   * @param string $path
   *   The URL path of the page to fetch.
   *
   * @return mixed The response from the path controller.
   */
  protected function getUrlResponse(string $path) {
    // Create the sub request.
    $sub_request = Request::create($path, 'GET');
    $type = HttpKernelInterface::SUB_REQUEST;
    $event = new RequestEvent($this->kernel, $sub_request, $type);
    $this->dispatcher->dispatch($event, KernelEvents::REQUEST);

    // Load the request controller.
    $controller = $this->controllerResolver->getController($sub_request);
    $event = new ControllerEvent($this->kernel, $controller, $sub_request, $type);
    $this->dispatcher->dispatch($event, KernelEvents::CONTROLLER);
    $controller = $event->getController();

    // Load the controller arguments.
    $arguments = $this->argumentResolver->getArguments($sub_request, $controller);
    $event = new ControllerArgumentsEvent($this->kernel, $controller, $arguments, $sub_request, $type);
    $this->dispatcher->dispatch($event, KernelEvents::CONTROLLER_ARGUMENTS);
    $controller = $event->getController();
    $arguments = $event->getArguments();

    // Call the controller.
    return $controller(...$arguments);
  }

}
